@@ -178,7 +178,7 @@ RegExpr() {
 
 static const char * DLL_CALLCONV
 MimeType() {
-	return "image/x-icon";
+	return "image/vnd.microsoft.icon";
 }
 
 static BOOL DLL_CALLCONV
@@ -269,6 +269,114 @@ PageCount(FreeImageIO *io, fi_handle handle, void *data) {
 
 // ----------------------------------------------------------
 
+static FIBITMAP*
+LoadStandardIcon(FreeImageIO *io, fi_handle handle, int flags, BOOL header_only) {
+	FIBITMAP *dib = NULL;
+
+	// load the BITMAPINFOHEADER
+	BITMAPINFOHEADER bmih;
+	io->read_proc(&bmih, sizeof(BITMAPINFOHEADER), 1, handle);
+#ifdef FREEIMAGE_BIGENDIAN
+	SwapInfoHeader(&bmih);
+#endif
+
+	// allocate the bitmap
+	int width  = bmih.biWidth;
+	int height = bmih.biHeight / 2; // height == xor + and mask
+	unsigned bit_count = bmih.biBitCount;
+	unsigned line   = CalculateLine(width, bit_count);
+	unsigned pitch  = CalculatePitch(line);
+
+	// allocate memory for one icon
+
+	dib = FreeImage_AllocateHeader(header_only, width, height, bit_count);
+
+	if (dib == NULL) {
+		return NULL;
+	}
+
+	if( bmih.biBitCount <= 8 ) {
+		// read the palette data
+		io->read_proc(FreeImage_GetPalette(dib), CalculateUsedPaletteEntries(bit_count) * sizeof(RGBQUAD), 1, handle);
+#if FREEIMAGE_COLORORDER == FREEIMAGE_COLORORDER_RGB
+		RGBQUAD *pal = FreeImage_GetPalette(dib);
+		for(unsigned i = 0; i < CalculateUsedPaletteEntries(bit_count); i++) {
+			INPLACESWAP(pal[i].rgbRed, pal[i].rgbBlue);
+		}
+#endif
+	}
+	
+	if(header_only) {
+		// header only mode
+		return dib;
+	}
+
+	// read the icon
+	io->read_proc(FreeImage_GetBits(dib), height * pitch, 1, handle);
+
+#ifdef FREEIMAGE_BIGENDIAN
+	if (bit_count == 16) {
+		for(int y = 0; y < height; y++) {
+			WORD *pixel = (WORD *)FreeImage_GetScanLine(dib, y);
+			for(int x = 0; x < width; x++) {
+				SwapShort(pixel);
+				pixel++;
+			}
+		}
+	}
+#endif
+#if FREEIMAGE_COLORORDER == FREEIMAGE_COLORORDER_RGB
+	if (bit_count == 24 || bit_count == 32) {
+		for(int y = 0; y < height; y++) {
+			BYTE *pixel = FreeImage_GetScanLine(dib, y);
+			for(int x = 0; x < width; x++) {
+				INPLACESWAP(pixel[0], pixel[2]);
+				pixel += (bit_count>>3);
+			}
+		}
+	}
+#endif
+	// bitmap has been loaded successfully!
+
+	// convert to 32bpp and generate an alpha channel
+	if((flags & ICO_MAKEALPHA) == ICO_MAKEALPHA) {
+		FIBITMAP *dib32 = FreeImage_ConvertTo32Bits(dib);
+		FreeImage_Unload(dib);
+
+		if (dib32 == NULL) {
+			return NULL;
+		}
+
+		int width_and	= WidthBytes(width);
+		BYTE *line_and	= (BYTE *)malloc(width_and);
+
+		if( line_and == NULL ) {
+			FreeImage_Unload(dib32);
+			return NULL;
+		}
+
+		//loop through each line of the AND-mask generating the alpha channel, invert XOR-mask
+		for(int y = 0; y < height; y++) {
+			RGBQUAD *quad = (RGBQUAD *)FreeImage_GetScanLine(dib32, y);
+			io->read_proc(line_and, width_and, 1, handle);
+			for(int x = 0; x < width; x++) {
+				quad->rgbReserved = (line_and[x>>3] & (0x80 >> (x & 0x07))) != 0 ? 0 : 0xFF;
+				if( quad->rgbReserved == 0 ) {
+					quad->rgbBlue ^= 0xFF;
+					quad->rgbGreen ^= 0xFF;
+					quad->rgbRed ^= 0xFF;
+				}
+				quad++;
+			}
+		}
+		free(line_and);
+
+		return dib32;
+	}
+
+	return dib;
+}
+
 static FIBITMAP * DLL_CALLCONV
 Load(FreeImageIO *io, fi_handle handle, int page, int flags, void *data) {
 	if (page == -1) {
@@ -301,117 +409,19 @@ Load(FreeImageIO *io, fi_handle handle, int page, int flags, void *data) {
 				io->seek_proc(handle, 0, SEEK_SET);
 				io->seek_proc(handle, icon_list[page].dwImageOffset, SEEK_CUR);
 
-				// Vista icon support
 				if((icon_list[page].bWidth == 0) && (icon_list[page].bHeight == 0)) {
+					// Vista icon support
 					dib = FreeImage_LoadFromHandle(FIF_PNG, io, handle, header_only ? FIF_LOAD_NOPIXELS : PNG_DEFAULT);
-					free(icon_list);
-					return dib;
+				}
+				else {
+					// standard icon support
+					// see http://msdn.microsoft.com/en-us/library/ms997538.aspx
+					dib = LoadStandardIcon(io, handle, flags, header_only);
 				}
 
 				free(icon_list);
 
-				// load the BITMAPINFOHEADER
-				BITMAPINFOHEADER bmih;
-				io->read_proc(&bmih, sizeof(BITMAPINFOHEADER), 1, handle);
-#ifdef FREEIMAGE_BIGENDIAN
-				SwapInfoHeader(&bmih);
-#endif
-
-				// allocate the bitmap
-				int width  = bmih.biWidth;
-				int height = bmih.biHeight / 2; // height == xor + and mask
-				unsigned bit_count = bmih.biBitCount;
-				unsigned line   = CalculateLine(width, bit_count);
-				unsigned pitch  = CalculatePitch(line);
-
-				// allocate memory for one icon
-
-				dib = FreeImage_AllocateHeader(header_only, width, height, bit_count);
-
-				if (dib == NULL) {
-					return NULL;
-				}
-
-				if( bmih.biBitCount <= 8 ) {
-					// read the palette data
-					io->read_proc(FreeImage_GetPalette(dib), CalculateUsedPaletteEntries(bit_count) * sizeof(RGBQUAD), 1, handle);
-#if FREEIMAGE_COLORORDER == FREEIMAGE_COLORORDER_RGB
-					RGBQUAD *pal = FreeImage_GetPalette(dib);
-					for(unsigned i = 0; i < CalculateUsedPaletteEntries(bit_count); i++) {
-						INPLACESWAP(pal[i].rgbRed, pal[i].rgbBlue);
-					}
-#endif
-				}
-				
-				if(header_only) {
-					// header only mode
-					return dib;
-				}
-
-				// read the icon
-				io->read_proc(FreeImage_GetBits(dib), height * pitch, 1, handle);
-
-#ifdef FREEIMAGE_BIGENDIAN
-				if (bit_count == 16) {
-					for(int y = 0; y < height; y++) {
-						WORD *pixel = (WORD *)FreeImage_GetScanLine(dib, y);
-						for(int x = 0; x < width; x++) {
-							SwapShort(pixel);
-							pixel++;
-						}
-					}
-				}
-#endif
-#if FREEIMAGE_COLORORDER == FREEIMAGE_COLORORDER_RGB
-				if (bit_count == 24 || bit_count == 32) {
-					for(int y = 0; y < height; y++) {
-						BYTE *pixel = FreeImage_GetScanLine(dib, y);
-						for(int x = 0; x < width; x++) {
-							INPLACESWAP(pixel[0], pixel[2]);
-							pixel += (bit_count>>3);
-						}
-					}
-				}
-#endif
-				// bitmap has been loaded successfully!
-
-				// convert to 32bpp and generate an alpha channel
-				if((flags & ICO_MAKEALPHA) == ICO_MAKEALPHA) {
-					FIBITMAP *dib32 = FreeImage_ConvertTo32Bits(dib);
-					FreeImage_Unload(dib);
-
-					if (dib32 == NULL) {
-						return NULL;
-					}
-
-					int width_and	= WidthBytes(width);
-					BYTE *line_and	= (BYTE *)malloc(width_and);
-
-					if( line_and == NULL ) {
-						FreeImage_Unload(dib32);
-						return NULL;
-					}
-
-					//loop through each line of the AND-mask generating the alpha channel, invert XOR-mask
-					for(int y = 0; y < height; y++) {
-						RGBQUAD *quad = (RGBQUAD *)FreeImage_GetScanLine(dib32, y);
-						io->read_proc(line_and, width_and, 1, handle);
-						for(int x = 0; x < width; x++) {
-							quad->rgbReserved = (line_and[x>>3] & (0x80 >> (x & 0x07))) != 0 ? 0 : 0xFF;
-							if( quad->rgbReserved == 0 ) {
-								quad->rgbBlue ^= 0xFF;
-								quad->rgbGreen ^= 0xFF;
-								quad->rgbRed ^= 0xFF;
-							}
-							quad++;
-						}
-					}
-					free(line_and);
-
-					return dib32;
-				}
-
-				return (FIBITMAP *)dib;
+				return dib;
 
 			} else {
 				free(icon_list);
@@ -425,37 +435,241 @@ Load(FreeImageIO *io, fi_handle handle, int page, int flags, void *data) {
 	return NULL;
 }
 
+// ----------------------------------------------------------
+
+static BOOL 
+SaveStandardIcon(FreeImageIO *io, FIBITMAP *dib, fi_handle handle) {
+	BITMAPINFOHEADER *bmih = NULL;
+
+	// write the BITMAPINFOHEADER
+	bmih = FreeImage_GetInfoHeader(dib);
+	bmih->biHeight *= 2;	// height == xor + and mask
+#ifdef FREEIMAGE_BIGENDIAN
+	SwapInfoHeader(bmih);
+#endif
+	io->write_proc(bmih, sizeof(BITMAPINFOHEADER), 1, handle);
+#ifdef FREEIMAGE_BIGENDIAN
+	SwapInfoHeader(bmih);
+#endif
+	bmih->biHeight /= 2;
+
+	// write the palette data
+	if (FreeImage_GetPalette(dib) != NULL) {
+		RGBQUAD *pal = FreeImage_GetPalette(dib);
+		FILE_BGRA bgra;
+		for(unsigned i = 0; i < FreeImage_GetColorsUsed(dib); i++) {
+			bgra.b = pal[i].rgbBlue;
+			bgra.g = pal[i].rgbGreen;
+			bgra.r = pal[i].rgbRed;
+			bgra.a = pal[i].rgbReserved;
+			io->write_proc(&bgra, sizeof(FILE_BGRA), 1, handle);
+		}
+	}
+
+	// write the bits
+	int width			= bmih->biWidth;
+	int height			= bmih->biHeight;
+	unsigned bit_count	= bmih->biBitCount;
+	unsigned line		= CalculateLine(width, bit_count);
+	unsigned pitch		= CalculatePitch(line);
+	int size_xor		= height * pitch;
+	int size_and		= height * WidthBytes(width);
+
+	// XOR mask
+#ifdef FREEIMAGE_BIGENDIAN
+	if (bit_count == 16) {
+		WORD pixel;
+		for(unsigned y = 0; y < FreeImage_GetHeight(dib); y++) {
+			BYTE *line = FreeImage_GetScanLine(dib, y);
+			for(unsigned x = 0; x < FreeImage_GetWidth(dib); x++) {
+				pixel = ((WORD *)line)[x];
+				SwapShort(&pixel);
+				if (io->write_proc(&pixel, sizeof(WORD), 1, handle) != 1)
+					return FALSE;
+			}
+		}
+	} else
+#endif
+#if FREEIMAGE_COLORORDER == FREEIMAGE_COLORORDER_RGB
+	if (bit_count == 24) {
+		FILE_BGR bgr;
+		for(unsigned y = 0; y < FreeImage_GetHeight(dib); y++) {
+			BYTE *line = FreeImage_GetScanLine(dib, y);
+			for(unsigned x = 0; x < FreeImage_GetWidth(dib); x++) {
+				RGBTRIPLE *triple = ((RGBTRIPLE *)line)+x;
+				bgr.b = triple->rgbtBlue;
+				bgr.g = triple->rgbtGreen;
+				bgr.r = triple->rgbtRed;
+				if (io->write_proc(&bgr, sizeof(FILE_BGR), 1, handle) != 1)
+					return FALSE;
+			}
+		}
+	} else if (bit_count == 32) {
+		FILE_BGRA bgra;
+		for(unsigned y = 0; y < FreeImage_GetHeight(dib); y++) {
+			BYTE *line = FreeImage_GetScanLine(dib, y);
+			for(unsigned x = 0; x < FreeImage_GetWidth(dib); x++) {
+				RGBQUAD *quad = ((RGBQUAD *)line)+x;
+				bgra.b = quad->rgbBlue;
+				bgra.g = quad->rgbGreen;
+				bgra.r = quad->rgbRed;
+				bgra.a = quad->rgbReserved;
+				if (io->write_proc(&bgra, sizeof(FILE_BGRA), 1, handle) != 1)
+					return FALSE;
+			}
+		}
+	} else
+#endif
+#if defined(FREEIMAGE_BIGENDIAN) || FREEIMAGE_COLORORDER == FREEIMAGE_COLORORDER_RGB
+	{
+#endif
+		BYTE *xor_mask = FreeImage_GetBits(dib);
+		io->write_proc(xor_mask, size_xor, 1, handle);
+#if defined(FREEIMAGE_BIGENDIAN) || FREEIMAGE_COLORORDER == FREEIMAGE_COLORORDER_RGB
+	}
+#endif
+	// AND mask
+	BYTE *and_mask = (BYTE*)malloc(size_and);
+	if(!and_mask) {
+		return FALSE;
+	}
+
+	if(FreeImage_IsTransparent(dib)) {
+
+		if(bit_count == 32) {
+			// create the AND mask from the alpha channel
+
+			int width_and  = WidthBytes(width);
+			BYTE *and_bits = and_mask;
+
+			// clear the mask
+			memset(and_mask, 0, size_and);
+
+			for(int y = 0; y < height; y++) {
+				RGBQUAD *bits = (RGBQUAD*)FreeImage_GetScanLine(dib, y);
+
+				for(int x = 0; x < width; x++) {
+					if(bits[x].rgbReserved != 0xFF) {
+						// set any transparent color to full transparency
+						and_bits[x >> 3] |= (0x80 >> (x & 0x7)); 
+					}
+				}
+
+				and_bits += width_and;
+			}
+		} 
+		else if(bit_count <= 8) {
+			// create the AND mask from the transparency table
+
+			BYTE *trns = FreeImage_GetTransparencyTable(dib);
+
+			int width_and  = WidthBytes(width);
+			BYTE *and_bits = and_mask;
+
+			// clear the mask
+			memset(and_mask, 0, size_and);
+
+			switch(FreeImage_GetBPP(dib)) {
+				case 1:
+				{
+					for(int y = 0; y < height; y++) {
+						BYTE *bits = (BYTE*)FreeImage_GetScanLine(dib, y);
+						for(int x = 0; x < width; x++) {
+							// get pixel at (x, y)
+							BYTE index = (bits[x >> 3] & (0x80 >> (x & 0x07))) != 0;
+							if(trns[index] != 0xFF) {
+								// set any transparent color to full transparency
+								and_bits[x >> 3] |= (0x80 >> (x & 0x7)); 
+							}
+						}
+						and_bits += width_and;
+					}
+				}
+				break;
+
+				case 4:
+				{
+					for(int y = 0; y < height; y++) {
+						BYTE *bits = (BYTE*)FreeImage_GetScanLine(dib, y);
+						for(int x = 0; x < width; x++) {
+							// get pixel at (x, y)
+							BYTE shift = (BYTE)((1 - x % 2) << 2);
+							BYTE index = (bits[x >> 1] & (0x0F << shift)) >> shift;
+							if(trns[index] != 0xFF) {
+								// set any transparent color to full transparency
+								and_bits[x >> 3] |= (0x80 >> (x & 0x7)); 
+							}
+						}
+						and_bits += width_and;
+					}
+				}
+				break;
+
+				case 8:
+				{
+					for(int y = 0; y < height; y++) {
+						BYTE *bits = (BYTE*)FreeImage_GetScanLine(dib, y);
+						for(int x = 0; x < width; x++) {
+							// get pixel at (x, y)
+							BYTE index = bits[x];
+							if(trns[index] != 0xFF) {
+								// set any transparent color to full transparency
+								and_bits[x >> 3] |= (0x80 >> (x & 0x7)); 
+							}
+						}
+						and_bits += width_and;
+					}
+				}
+				break;
+
+			}
+		}
+	}
+	else {
+		// empty AND mask
+		memset(and_mask, 0, size_and);
+	}
+
+	io->write_proc(and_mask, size_and, 1, handle);
+	free(and_mask);
+
+	return TRUE;
+}
+
 static BOOL DLL_CALLCONV
 Save(FreeImageIO *io, FIBITMAP *dib, fi_handle handle, int page, int flags, void *data) {
+	ICONHEADER *icon_header = NULL;
+	std::vector<FIBITMAP*> vPages;
 	int k;
 
-	if(dib) {
-		// check format limits
-		unsigned w = FreeImage_GetWidth(dib);
-		unsigned h = FreeImage_GetHeight(dib);
-		if((w < 16) || (w > 256) || (h < 16) || (h > 256)) {
-			FreeImage_OutputMessageProc(s_format_id, "Unsupported icon size");
-			return FALSE;
-		}
-	} else {
+	if(!dib || !handle || !data) {
+		return FALSE;
+	}
+
+	// check format limits
+	unsigned w = FreeImage_GetWidth(dib);
+	unsigned h = FreeImage_GetHeight(dib);
+	if((w < 16) || (w > 256) || (h < 16) || (h > 256) || (w != h)) {
+		FreeImage_OutputMessageProc(s_format_id, "Unsupported icon size: width x height = %d x %d", w, h);
 		return FALSE;
 	}
 
 	if (page == -1) {
 		page = 0;
 	}
-
+	
 	// get the icon header
-	ICONHEADER *icon_header = (ICONHEADER*)data;
+	icon_header = (ICONHEADER*)data;
 
-	if(icon_header) {
-
-		std::vector<FIBITMAP*> vPages;
-		FIBITMAP  *icon_dib = NULL;
+	try {
+		FIBITMAP *icon_dib = NULL;
 
 		// load all icons
 		for(k = 0; k < icon_header->idCount; k++) {
 			icon_dib = Load(io, handle, k, flags, data);
+			if(!icon_dib) {
+				throw FI_MSG_ERROR_DIB_MEMORY;
+			}
 			vPages.push_back(icon_dib);
 		}
 
@@ -476,18 +690,23 @@ Save(FreeImageIO *io, FIBITMAP *dib, fi_handle handle, int page, int flags, void
 
 		// write all icons
 		// ...
-
+		
 		// save the icon descriptions
+
 		ICONDIRENTRY *icon_list = (ICONDIRENTRY *)malloc(icon_header->idCount * sizeof(ICONDIRENTRY));
+		if(!icon_list) {
+			throw FI_MSG_ERROR_MEMORY;
+		}
 		memset(icon_list, 0, icon_header->idCount * sizeof(ICONDIRENTRY));
-		BITMAPINFOHEADER *bmih;
+
 		for(k = 0; k < icon_header->idCount; k++) {
 			icon_dib = (FIBITMAP*)vPages[k];
 
 			// convert internal format to ICONDIRENTRY
-			bmih = FreeImage_GetInfoHeader(icon_dib);
-			icon_list[k].bWidth			= (BYTE)bmih->biWidth;
-			icon_list[k].bHeight		= (BYTE)bmih->biHeight;
+			// take into account Vista icons whose size is 256x256
+			const BITMAPINFOHEADER *bmih = FreeImage_GetInfoHeader(icon_dib);
+			icon_list[k].bWidth			= (bmih->biWidth > 255)  ? 0 : (BYTE)bmih->biWidth;
+			icon_list[k].bHeight		= (bmih->biHeight > 255) ? 0 : (BYTE)bmih->biHeight;
 			icon_list[k].bReserved		= 0;
 			icon_list[k].wPlanes		= bmih->biPlanes;
 			icon_list[k].wBitCount		= bmih->biBitCount;
@@ -496,210 +715,50 @@ Save(FreeImageIO *io, FIBITMAP *dib, fi_handle handle, int page, int flags, void
 			} else {
 				icon_list[k].bColorCount = (BYTE)(1 << (icon_list[k].wPlanes * icon_list[k].wBitCount));
 			}
+			// initial guess (correct only for standard icons)
 			icon_list[k].dwBytesInRes	= CalculateImageSize(icon_dib);
 			icon_list[k].dwImageOffset = CalculateImageOffset(vPages, k);
 		}
+
+		// make a room for icon dir entries, until later update
+		const long directory_start = io->tell_proc(handle);
+		io->write_proc(icon_list, sizeof(ICONDIRENTRY) * icon_header->idCount, 1, handle);
+
+		// write the image bits for each image
+		
+		DWORD dwImageOffset = (DWORD)io->tell_proc(handle);
+
+		for(k = 0; k < icon_header->idCount; k++) {
+			icon_dib = (FIBITMAP*)vPages[k];
+			
+			if((icon_list[k].bWidth == 0) && (icon_list[k].bHeight == 0)) {
+				// Vista icon support
+				FreeImage_SaveToHandle(FIF_PNG, icon_dib, io, handle, PNG_DEFAULT);
+			}
+			else {
+				// standard icon support
+				// see http://msdn.microsoft.com/en-us/library/ms997538.aspx
+				SaveStandardIcon(io, icon_dib, handle);
+			}
+
+			// update ICONDIRENTRY members			
+			DWORD dwBytesInRes = (DWORD)io->tell_proc(handle) - dwImageOffset;
+			icon_list[k].dwImageOffset = dwImageOffset;
+			icon_list[k].dwBytesInRes  = dwBytesInRes;
+			dwImageOffset += dwBytesInRes;
+		}
+
+		// update the icon descriptions
+		const long current_pos = io->tell_proc(handle);
+		io->seek_proc(handle, directory_start, SEEK_SET);
 #ifdef FREEIMAGE_BIGENDIAN
 		SwapIconDirEntries(icon_list, icon_header->idCount);
 #endif
 		io->write_proc(icon_list, sizeof(ICONDIRENTRY) * icon_header->idCount, 1, handle);
+		io->seek_proc(handle, current_pos, SEEK_SET);
+
 		free(icon_list);
 
-		// write the image bits for each image
-		for(k = 0; k < icon_header->idCount; k++) {
-			icon_dib = (FIBITMAP*)vPages[k];
-
-			// write the BITMAPINFOHEADER
-			bmih = FreeImage_GetInfoHeader(icon_dib);
-			bmih->biHeight *= 2;	// height == xor + and mask
-#ifdef FREEIMAGE_BIGENDIAN
-			SwapInfoHeader(bmih);
-#endif
-			io->write_proc(bmih, sizeof(BITMAPINFOHEADER), 1, handle);
-#ifdef FREEIMAGE_BIGENDIAN
-			SwapInfoHeader(bmih);
-#endif
-			bmih->biHeight /= 2;
-
-			// write the palette data
-			if (FreeImage_GetPalette(icon_dib) != NULL) {
-				RGBQUAD *pal = FreeImage_GetPalette(icon_dib);
-				FILE_BGRA bgra;
-				for(unsigned i = 0; i < FreeImage_GetColorsUsed(icon_dib); i++) {
-					bgra.b = pal[i].rgbBlue;
-					bgra.g = pal[i].rgbGreen;
-					bgra.r = pal[i].rgbRed;
-					bgra.a = pal[i].rgbReserved;
-					io->write_proc(&bgra, sizeof(FILE_BGRA), 1, handle);
-				}
-			}
-
-			// write the bits
-			int width			= bmih->biWidth;
-			int height			= bmih->biHeight;
-			unsigned bit_count	= bmih->biBitCount;
-			unsigned line		= CalculateLine(width, bit_count);
-			unsigned pitch		= CalculatePitch(line);
-			int size_xor		= height * pitch;
-			int size_and		= height * WidthBytes(width);
-
-			// XOR mask
-#ifdef FREEIMAGE_BIGENDIAN
-			if (bit_count == 16) {
-				WORD pixel;
-				for(unsigned y = 0; y < FreeImage_GetHeight(icon_dib); y++) {
-					BYTE *line = FreeImage_GetScanLine(icon_dib, y);
-					for(unsigned x = 0; x < FreeImage_GetWidth(icon_dib); x++) {
-						pixel = ((WORD *)line)[x];
-						SwapShort(&pixel);
-						if (io->write_proc(&pixel, sizeof(WORD), 1, handle) != 1)
-							return FALSE;
-					}
-				}
-			} else
-#endif
-#if FREEIMAGE_COLORORDER == FREEIMAGE_COLORORDER_RGB
-			if (bit_count == 24) {
-				FILE_BGR bgr;
-				for(unsigned y = 0; y < FreeImage_GetHeight(icon_dib); y++) {
-					BYTE *line = FreeImage_GetScanLine(icon_dib, y);
-					for(unsigned x = 0; x < FreeImage_GetWidth(icon_dib); x++) {
-						RGBTRIPLE *triple = ((RGBTRIPLE *)line)+x;
-						bgr.b = triple->rgbtBlue;
-						bgr.g = triple->rgbtGreen;
-						bgr.r = triple->rgbtRed;
-						if (io->write_proc(&bgr, sizeof(FILE_BGR), 1, handle) != 1)
-							return FALSE;
-					}
-				}
-			} else if (bit_count == 32) {
-				FILE_BGRA bgra;
-				for(unsigned y = 0; y < FreeImage_GetHeight(icon_dib); y++) {
-					BYTE *line = FreeImage_GetScanLine(icon_dib, y);
-					for(unsigned x = 0; x < FreeImage_GetWidth(icon_dib); x++) {
-						RGBQUAD *quad = ((RGBQUAD *)line)+x;
-						bgra.b = quad->rgbBlue;
-						bgra.g = quad->rgbGreen;
-						bgra.r = quad->rgbRed;
-						bgra.a = quad->rgbReserved;
-						if (io->write_proc(&bgra, sizeof(FILE_BGRA), 1, handle) != 1)
-							return FALSE;
-					}
-				}
-			} else
-#endif
-#if defined(FREEIMAGE_BIGENDIAN) || FREEIMAGE_COLORORDER == FREEIMAGE_COLORORDER_RGB
-			{
-#endif
-				BYTE *xor_mask = FreeImage_GetBits(icon_dib);
-				io->write_proc(xor_mask, size_xor, 1, handle);
-#if defined(FREEIMAGE_BIGENDIAN) || FREEIMAGE_COLORORDER == FREEIMAGE_COLORORDER_RGB
-			}
-#endif
-			// AND mask
-			BYTE *and_mask = (BYTE*)malloc(size_and);
-
-			if(FreeImage_IsTransparent(dib)) {
-
-				if(bit_count == 32) {
-					// create the AND mask from the alpha channel
-
-					int width_and  = WidthBytes(width);
-					BYTE *and_bits = and_mask;
-
-					// clear the mask
-					memset(and_mask, 0, size_and);
-
-					for(int y = 0; y < height; y++) {
-						RGBQUAD *bits = (RGBQUAD*)FreeImage_GetScanLine(dib, y);
-
-						for(int x = 0; x < width; x++) {
-							if(bits[x].rgbReserved != 0xFF) {
-								// set any transparent color to full transparency
-								and_bits[x >> 3] |= (0x80 >> (x & 0x7)); 
-							}
-						}
-
-						and_bits += width_and;
-					}
-				} 
-				else if(bit_count <= 8) {
-					// create the AND mask from the transparency table
-
-					BYTE *trns = FreeImage_GetTransparencyTable(dib);
-
-					int width_and  = WidthBytes(width);
-					BYTE *and_bits = and_mask;
-
-					// clear the mask
-					memset(and_mask, 0, size_and);
-
-					switch(FreeImage_GetBPP(dib)) {
-						case 1:
-						{
-							for(int y = 0; y < height; y++) {
-								BYTE *bits = (BYTE*)FreeImage_GetScanLine(dib, y);
-								for(int x = 0; x < width; x++) {
-									// get pixel at (x, y)
-									BYTE index = (bits[x >> 3] & (0x80 >> (x & 0x07))) != 0;
-									if(trns[index] != 0xFF) {
-										// set any transparent color to full transparency
-										and_bits[x >> 3] |= (0x80 >> (x & 0x7)); 
-									}
-								}
-								and_bits += width_and;
-							}
-						}
-						break;
-
-						case 4:
-						{
-							for(int y = 0; y < height; y++) {
-								BYTE *bits = (BYTE*)FreeImage_GetScanLine(dib, y);
-								for(int x = 0; x < width; x++) {
-									// get pixel at (x, y)
-									BYTE shift = (BYTE)((1 - x % 2) << 2);
-									BYTE index = (bits[x >> 1] & (0x0F << shift)) >> shift;
-									if(trns[index] != 0xFF) {
-										// set any transparent color to full transparency
-										and_bits[x >> 3] |= (0x80 >> (x & 0x7)); 
-									}
-								}
-								and_bits += width_and;
-							}
-						}
-						break;
-
-						case 8:
-						{
-							for(int y = 0; y < height; y++) {
-								BYTE *bits = (BYTE*)FreeImage_GetScanLine(dib, y);
-								for(int x = 0; x < width; x++) {
-									// get pixel at (x, y)
-									BYTE index = bits[x];
-									if(trns[index] != 0xFF) {
-										// set any transparent color to full transparency
-										and_bits[x >> 3] |= (0x80 >> (x & 0x7)); 
-									}
-								}
-								and_bits += width_and;
-							}
-						}
-						break;
-
-					}
-				}
-			}
-			else {
-				// empty AND mask
-				memset(and_mask, 0, size_and);
-			}
-
-			io->write_proc(and_mask, size_and, 1, handle);
-			free(and_mask);
-
-		}
-
 		// free the vector class
 		for(k = 0; k < icon_header->idCount; k++) {
 			icon_dib = (FIBITMAP*)vPages[k];
@@ -707,9 +766,16 @@ Save(FreeImageIO *io, FIBITMAP *dib, fi_handle handle, int page, int flags, void
 		}
 
 		return TRUE;
-	}
 
-	return FALSE;
+	} catch(const char *text) {
+		// free the vector class
+		for(size_t k = 0; k < vPages.size(); k++) {
+			FIBITMAP *icon_dib = (FIBITMAP*)vPages[k];
+			FreeImage_Unload(icon_dib);
+		}
+		FreeImage_OutputMessageProc(s_format_id, text);
+		return FALSE;
+	}
 }
 
 // ==========================================================
