@@ -96,6 +96,12 @@ Allocate a FIBITMAP of type FIT_BITMAP, with possibly no pixel data
 */
 DLL_API FIBITMAP * DLL_CALLCONV FreeImage_AllocateHeader(BOOL header_only, int width, int height, int bpp, unsigned red_mask FI_DEFAULT(0), unsigned green_mask FI_DEFAULT(0), unsigned blue_mask FI_DEFAULT(0));
 
+/**
+Helper for 16-bit FIT_BITMAP
+@see FreeImage_GetRGBMasks
+*/
+DLL_API BOOL DLL_CALLCONV FreeImage_HasRGBMasks(FIBITMAP *dib);
+
 #if defined(__cplusplus)
 }
 #endif
@@ -141,12 +147,12 @@ typedef struct tagFILE_BGR {
 // ==========================================================
 
 /// Max function
-template <class T> T MAX(T a, T b) {
+template <class T> T MAX(const T &a, const T &b) {
 	return (a > b) ? a: b;
 }
 
 /// Min function
-template <class T> T MIN(T a, T b) {
+template <class T> T MIN(const T &a, const T &b) {
 	return (a < b) ? a: b;
 }
 
@@ -156,7 +162,7 @@ template <class T> void INPLACESWAP(T& a, T& b) {
 }
 
 /// Clamp function
-template <class T> T CLAMP(T value, T min_value, T max_value) {
+template <class T> T CLAMP(const T &value, const T &min_value, const T &max_value) {
 	return ((value < min_value) ? min_value : (value > max_value) ? max_value : value);
 }
 
@@ -252,7 +258,7 @@ CalculateUsedBits(int bits) {
 
 inline unsigned
 CalculateLine(unsigned width, unsigned bitdepth) {
-	return ((width * bitdepth) + 7) / 8;
+	return (unsigned)( ((unsigned long long)width * bitdepth + 7) / 8 );
 }
 
 inline unsigned
@@ -276,7 +282,7 @@ CalculateScanLine(unsigned char *bits, unsigned pitch, int scanline) {
 // ----------------------------------------------------------
 
 /**
-Fast generic assign (faster then for loop)
+Fast generic assign (faster than for loop)
 @param dst Destination pixel
 @param src Source pixel
 @param bytesperpixel # of bytes per pixel
@@ -456,8 +462,8 @@ A Standard Default Color Space for the Internet - sRGB.
 #define RGB565(b, g, r) ((((b) >> 3) << FI16_565_BLUE_SHIFT) | (((g) >> 2) << FI16_565_GREEN_SHIFT) | (((r) >> 3) << FI16_565_RED_SHIFT))
 #define RGB555(b, g, r) ((((b) >> 3) << FI16_555_BLUE_SHIFT) | (((g) >> 3) << FI16_555_GREEN_SHIFT) | (((r) >> 3) << FI16_555_RED_SHIFT))
 
-#define FORMAT_RGB565(dib) ((FreeImage_GetRedMask(dib) == FI16_565_RED_MASK) &&(FreeImage_GetGreenMask(dib) == FI16_565_GREEN_MASK) &&(FreeImage_GetBlueMask(dib) == FI16_565_BLUE_MASK))
-#define RGBQUAD_TO_WORD(dib, color) (FORMAT_RGB565(dib) ? RGB565((color)->rgbBlue, (color)->rgbGreen, (color)->rgbRed) : RGB555((color)->rgbBlue, (color)->rgbGreen, (color)->rgbRed))
+#define IS_FORMAT_RGB565(dib) ((FreeImage_GetRedMask(dib) == FI16_565_RED_MASK) && (FreeImage_GetGreenMask(dib) == FI16_565_GREEN_MASK) && (FreeImage_GetBlueMask(dib) == FI16_565_BLUE_MASK))
+#define RGBQUAD_TO_WORD(dib, color) (IS_FORMAT_RGB565(dib) ? RGB565((color)->rgbBlue, (color)->rgbGreen, (color)->rgbRed) : RGB555((color)->rgbBlue, (color)->rgbGreen, (color)->rgbRed))
 
 #define CREATE_GREYSCALE_PALETTE(palette, entries) \
 	for (unsigned i = 0, v = 0; i < entries; i++, v += 0x00FFFFFF / (entries - 1)) { \
@@ -474,7 +480,7 @@ A Standard Default Color Space for the Internet - sRGB.
 // ==========================================================
 
 static const char *FI_MSG_ERROR_MEMORY = "Memory allocation failed";
-static const char *FI_MSG_ERROR_DIB_MEMORY = "DIB allocation failed, probably caused by an invalid image";
+static const char *FI_MSG_ERROR_DIB_MEMORY = "DIB allocation failed, maybe caused by an invalid image size or by a lack of memory";
 static const char *FI_MSG_ERROR_PARSING = "Parsing error";
 static const char *FI_MSG_ERROR_MAGIC_NUMBER = "Invalid magic number";
 static const char *FI_MSG_ERROR_UNSUPPORTED_FORMAT = "Unsupported format";
